/*
 * @(#)MethodRepository.java	1.3 05/11/17
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package sun.reflect.generics.repository;


import java.lang.reflect.Type;
import sun.reflect.generics.factory.GenericsFactory;
import sun.reflect.generics.visitor.Reifier;



/**
 * This class represents the generic type information for a method.
 * The code is not dependent on a particular reflective implementation.
 * It is designed to be used unchanged by at least core reflection and JDI.
 */
public class MethodRepository extends ConstructorRepository {

    private Type returnType; // caches the generic return type info

 // private, to enforce use of static factory
    private MethodRepository(String rawSig, GenericsFactory f) {
      super(rawSig, f);
    }

    /**
     * Static factory method.
     * @param rawSig - the generic signature of the reflective object
     * that this repository is servicing
     * @param f - a factory that will provide instances of reflective
     * objects when this repository converts its AST
     * @return a <tt>MethodRepository</tt> that manages the generic type
     * information represented in the signature <tt>rawSig</tt>
     */
    public static MethodRepository make(String rawSig, GenericsFactory f) {
	return new MethodRepository(rawSig, f);
    }

    // public API

    public Type getReturnType() {
	if (returnType == null) { // lazily initialize return type
	    Reifier r = getReifier(); // obtain visitor
	    // Extract return type subtree from AST and reify
	    getTree().getReturnType().accept(r);
	    // extract result from visitor and cache it
	    returnType = r.getResult();
	    }
	return returnType; // return cached result
    }


}
