/*
 * @(#)TimeZoneNamesBundle.java	1.3 05/11/17
 */

/*
 * Portions Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * (C) Copyright Taligent, Inc. 1996, 1997 - All Rights Reserved
 * (C) Copyright IBM Corp. 1996 - 1998 - All Rights Reserved
 *
 * The original version of this source code and documentation
 * is copyrighted and owned by Taligent, Inc., a wholly-owned
 * subsidiary of IBM. These materials are provided under terms
 * of a License Agreement between Taligent and Sun. This technology
 * is protected by multiple US and International patents.
 *
 * This notice and attribution to Taligent may not be removed.
 * Taligent is a registered trademark of Taligent, Inc.
 *
 */

package sun.util.resources;

import java.util.Map;
import java.util.LinkedHashMap;

/**
 * Subclass of <code>ResourceBundle</code> with special
 * functionality for time zone names. The additional functionality:
 * <ul>
 * <li>Preserves the order of entries in the <code>getContents</code>
 *     array for the enumeration returned by <code>getKeys</code>.
 * <li>Inserts the time zone ID (the key of the bundle entries) into
 *     the string arrays returned by <code>handleGetObject</code>.
 * <ul>
 * All <code>TimeZoneNames</code> resource bundles must extend this
 * class and implement the <code>getContents</code> method.
 */
public abstract class TimeZoneNamesBundle extends OpenListResourceBundle {

    /**
     * Maps time zone IDs to locale-specific names.
     * The value returned is an array of five strings:
     * <ul>
     * <li>The time zone ID (same as the key, not localized).
     * <li>The long name of the time zone in standard time (localized).
     * <li>The short name of the time zone in standard time (localized).
     * <li>The long name of the time zone in daylight savings time (localized).
     * <li>The short name of the time zone in daylight savings time (localized).
     * </ul>
     * The localized names come from the subclasses's
     * <code>getContents</code> implementations, while the time zone
     * ID is inserted into the returned array by this method.
     */
    public Object handleGetObject(String key) {
        String[] contents = (String[]) super.handleGetObject(key);
	if (contents == null) { 
	    return null;	
	}

	int clen = contents.length;
	String[] tmpobj = new String[clen+1];
	tmpobj[0] = key;
	for (int i = 0; i < clen; i++) {
	    tmpobj[i+1] = contents[i];
	}
	return tmpobj;
    }

    /**
     * Use LinkedHashMap to preserve order of bundle entries.
     */
    protected Map createMap(int size) {
        return new LinkedHashMap(size);
    }
    
    /**
     * Provides key/value mappings for a specific
     * resource bundle. Each entry of the array
     * returned must be an array with two elements:
     * <ul>
     * <li>The key, which must be a string.
     * <li>The value, which must be an array of
     *     four strings:
     *     <ul>
     *     <li>The long name of the time zone in standard time.
     *     <li>The short name of the time zone in standard time.
     *     <li>The long name of the time zone in daylight savings time.
     *     <li>The short name of the time zone in daylight savings time.
     *     </ul>
     * </ul>
     */
    protected abstract Object[][] getContents();
}
